# SPDX-FileCopyrightText: 2023-2024 oeykmiih
# SPDX-License-Identifier: GPL-3.0-or-later
import bpy

from . import data


def obt(name, force=False, parent=None, local=False, overwrite=None):
    if local:
        blcol = data.obt(
            bpy.data.collections,
            name,
            scope=bpy.context.scene.collection.children_recursive,
            force=force,
            overwrite=overwrite,
        )
    else:
        blcol = data.obt(bpy.data.collections, name, force=force, overwrite=overwrite)

    if parent is not None and blcol is not None and name not in parent.children:
        parent.children.link(blcol)
    return blcol


def empty(blcol, objects=False, recursive=False):
    collections = [c for c in blcol.children]

    while collections:
        child = collections.pop()
        if recursive:
            empty(child, recursive=True, objects=objects)
        blcol.children.unlink(child)

    if objects:
        objects = [o for o in blcol.objects]
        while objects:
            blcol.objects.unlink(objects.pop())


def rget_collection_path(blcol, node, parent_path=()):
    for subnode in node.children:
        current_path = parent_path + (subnode,)

        if blcol == subnode:
            yield current_path
        elif blcol in subnode.children_recursive:
            yield from rget_collection_path(blcol, subnode, current_path)


def get_paths_from_scene(blcol, scene):
    return rget_collection_path(blcol, scene.collection)


def get_layer_collections(blcol, view_layer):
    scene = view_layer.id_data
    for path in get_paths_from_scene(blcol, scene):
        layer_collection = view_layer.layer_collection
        for collection in path:
            layer_collection = layer_collection.children[collection.name]
        yield layer_collection


def exclude_from_viewlayer(blcol, view_layer):
    for layer_collection in get_layer_collections(blcol, view_layer):
        layer_collection.exclude = True
