# SPDX-FileCopyrightText: 2023-2024 oeykmiih
# SPDX-License-Identifier: GPL-3.0-or-later

from . import ops


def alert(layout, text="", align=True, enabled=True):
    widget = layout.column(align=align)
    widget.alert = True
    widget.enabled = enabled
    widget.operator(
        ops.UTILS_OT_placeholder.bl_idname,
        text=text,
    )
    return widget


def empty(layout, icon="BLANK1", depress=False, align=True, emboss=False, enabled=True):
    widget = layout.row(align=align)
    widget.enabled = enabled
    widget.operator(
        ops.UTILS_OT_placeholder.bl_idname,
        text="",
        depress=depress,
        emboss=emboss,
        icon=icon,
    )
    return widget


def label(layout, text="", depress=False, align=True, emboss=True, enabled=True):
    widget = layout.row(align=align)
    widget.enabled = enabled
    widget.operator(
        ops.UTILS_OT_placeholder.bl_idname,
        text=text,
        depress=depress,
        emboss=emboss,
    )
    return widget


def pie_panel(pie):
    row = pie.row()
    row.ui_units_x = 7.0
    return row.column()


def prop_safe(layout, object, key, **kwargs):
    if getattr(object, key, None) is not None:
        layout.prop(object, key, **kwargs)


def section(layout, text=""):
    layout.separator()
    section = layout.box().column(align=True)

    if text != "":
        section.label(text=text)
    return section


def split(layout, text="", align=True, enabled=True, factor=0.4):
    row = layout.row(align=True).split(factor=factor, align=True)
    row.enabled = enabled
    row.use_property_split = False
    label = row.row()
    label.alignment = "RIGHT"
    label.label(text=text)

    sub = row.row(align=True)
    return sub


def split2(
    layout, label_alignment="RIGHT", factor=0.4, align=False, enabled=True, alert=False
):
    row = layout.row(align=True).split(factor=factor, align=True)
    row.enabled = enabled
    row.alert = alert
    label = row.row()
    label.alignment = label_alignment

    content = row.row(align=align)
    return label, content


def split3(layout, factor=0.1, align=False):
    split1 = layout.split(factor=factor, align=align)
    left = split1.row(align=align)
    split2 = split1.row(align=True).split(
        factor=1.0 - factor / (1.0 - factor), align=align
    )
    split2.alignment = "RIGHT"
    content = split2.row(align=align)
    right = split2.row(align=align)
    return left, content, right


def submodules_preferences(preferences, layout, modules):
    for name in (name for name, module in modules.items() if hasattr(module, "draw")):
        module = modules[name]
        box = layout.box()
        box.label(text=name.replace("_", " ").title())
        properties = getattr(preferences, name)
        module.draw(properties, box)
    return None
