# SPDX-FileCopyrightText: 2023-2024 oeykmiih
# SPDX-License-Identifier: GPL-3.0-or-later


# CREDIT: https://stackoverflow.com/a/26209900
class formatter(object):
    def __init__(self):
        self.types = {}
        self.htchar = "        "
        self.lfchar = "\n"
        self.indent = 0
        self.set_formater(object, self.__class__.format_object)
        self.set_formater(dict, self.__class__.format_dict)
        self.set_formater(list, self.__class__.format_list)
        self.set_formater(tuple, self.__class__.format_tuple)

    def set_formater(self, obj, callback):
        self.types[obj] = callback

    def __call__(self, value, **args):
        for key in args:
            setattr(self, key, args[key])
        formater = self.types[type(value) if type(value) in self.types else object]
        return formater(self, value, self.indent)

    def print(self, value, **args):
        print(self.__call__(value))

    def format_object(self, value, indent):
        return repr(value)

    def format_dict(self, value, indent):
        items = [
            self.lfchar
            + self.htchar * (indent + 1)
            + repr(key)
            + ": "
            + (
                self.types[
                    type(value[key]) if type(value[key]) in self.types else object
                ]
            )(self, value[key], indent + 1)
            for key in value
        ]
        return "{%s}" % (",".join(items) + self.lfchar + self.htchar * indent)

    def format_list(self, value, indent):
        items = [
            self.lfchar
            + self.htchar * (indent + 1)
            + (self.types[type(item) if type(item) in self.types else object])(
                self, item, indent + 1
            )
            for item in value
        ]
        return "[%s]" % (",".join(items) + self.lfchar + self.htchar * indent)

    def format_tuple(self, value, indent):
        items = [
            self.lfchar
            + self.htchar * (indent + 1)
            + (self.types[type(item) if type(item) in self.types else object])(
                self, item, indent + 1
            )
            for item in value
        ]
        return "(%s)" % (",".join(items) + self.lfchar + self.htchar * indent)
